
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                    12F675.lkr                                       *
;    PPM_SUM                                                          *
;    Usart 115200                                                     *
;       4017 , 8  PPM 					  *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;    Pin1:       Vdd        + 5Volt                                   *
;    Channel-Output GPIO 0  Pin 7     (USART) LED    4017 clock (PPM) *
;    Channel-Output GPIO 1  Pin 6     (PPM SUM)   4017 reset (synhro) *
;    Signal-In/Out  GPIO 2  Pin 5 Dataline                            *
;    Signal-Input:  GPIO 3  Pin 4 Bind Pin                            *
;    Input:         GPIO 4  Pin 3 Oscillator Quarz                    *
;    Input:         GPIO 5  Pin 2 Oscillator Quarz                    *
;    Pin8:       Vss        Minus                                     *
;                                                                     *
;																	  *
;                                                                     *     
;                                                                     *     
;   _MCLRE_OFF                       				                  *     
;   _BODEN_ON  													      *
;**********************************************************************

	list      p=12f675		; list directive to define processor
	#include <p12f675.inc>	; processor specific variable definitions

	errorlevel  -302		; suppress message 302 from list file


;Quarz Oszillator mit mindestens 16 MHz (Hchstfrequenz bei 3.3 Volt Spannung) verwendet werden.
	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN_ON & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _HS_OSC  


;***** VARIABLE DEFINITIONS **********
			
;**************************************************************************************************************************			

#define		Prog_Level		"01"		;Die Programmversion kenntlich machen
#define		Sample_Time		.176 		;Dieser Wert ist korrekt !!! Read (Bind)-Data
#define 	Full_Bit_Time	.185		;Dieser Wert funktioniert!!!! Send Bind-Data

;**************************************************************************************************************************


#define 	InputLevel	My_Status,0
#define		BitFound	My_Status,1
#define		Send_Bind	My_Status,2
#define		Send		My_Status,3
#define		StartBit	My_Status,4
#define		StopBit		My_Status,5
#define		With_Startbit	My_Status,6	
#define		Checksum_Ok	My_Status,7

;**************************************************************************************************************************						
#define		LED				GPIO,0			;Anzeige LED "Bind complete" / "Online"
#define		Channel1		GPIO,1			;Ausgang fr Kanal1
#define		DataLine		GPIO,2			;Auf diesen Eingang die Dataline des Receivers legen
#define 	BindLine		GPIO,3			;Auf diesen Pin den Bind-Plug stecken
;---------------------------------------------------------------------------

; example of using Shared Uninitialized Data Section
INT_VAR		UDATA_SHR	0x20   
w_temp		RES     1		;0x20  variable used for context saving 
status_temp	RES     1		;0x21  variable used for context saving
My_Status	RES		1		;0x22
Bit_Count	RES		1		;0x23
Byte_Count	RES		1		;0x24
ShiftData	RES		1		;0x25
From_Adrs	RES		1		;0x26
To_Adrs		RES		1		;0x27
Inner		RES		1		;0x28
IInner		RES		1		;0x29
Outer		RES		1		;0x2a
Cksum_High	RES		1		;0x2b
Temp		RES		1		;0x2c
Byte1		RES		1		;0x2d
Byte2		RES		1		;0x2e
Byte3		RES		1		;0x2f
Byte4		RES		1		;0x30
Byte5		RES		1		;0x31
Byte6		RES		1		;0x32
Byte7		RES		1		;0x33
Byte8		RES		1		;0x34
Byte9		RES		1		;0x35
Byte10		RES		1		;0x36
Byte11		RES		1		;0x37
Byte12		RES		1		;0x38
Byte13		RES		1		;0x39
Byte14		RES		1		;0x3a
Byte15		RES		1		;0x3b
Byte16		RES		1		;0x3c
Byte17		RES		1		;0x3d
Byte18		RES		1		;0x3e
Copy_Byte1	RES		1		;0x3f
Copy_Byte2	RES		1		;0x40
Copy_Byte3	RES		1		;0x41
Copy_Byte4 	RES		1		;0x42
Copy_Byte5	RES		1		;0x43
Copy_Byte6	RES		1		;0x44
Copy_Byte7	RES		1		;0x45
Copy_Byte8	RES		1		;0x46
Copy_Byte9	RES		1		;0x47
Copy_Byte10	RES		1		;0x48
Copy_Byte11	RES		1		;0x49
Copy_Byte12	RES		1		;0x4a
Copy_Byte13	RES		1		;0x4b
Copy_Byte14	RES		1		;0x4c
Copy_Byte15	RES		1		;0x4d
Copy_Byte16	RES		1		;0x4e
My_Temp		RES		1
Selected_Channel	RES		1

bits_usart	RES		1
S_pause		RES		1
deb			RES		1
temp2		RES		1


;**********************************************************************
RESET_VECTOR	CODE	0x000	; processor reset vector
		goto    main			; go to beginning of program

;========================================================================
;************************************************************************
INT_VECTOR	CODE	0x004		; interrupt vector location
		movwf	w_temp			;copy W to temp-register
		swapf	STATUS,W		;swap status into W   Bank 0,1			
		bcf		STATUS,RP0		;change to Bank 0
		movwf	status_temp		; save STATUS to bank 0 register		

;		bsf		LED		;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!mein Synch Impuls for Scope
		nop						;don't change timing
		
		btfsc	Send_Bind
		goto	Send_Bit		;have to send binding information
		
		
;---------- Read Binding or channel Information ----------------------------
Read_Bit
		bsf		BitFound		;We have a bit weil wir einen Interrupt hatten. Entweder ist der Timer ausgetimed oder 

		movlw	1
		subwf	Bit_Count,W		;Ergebnis in W
		btfsc	STATUS,Z		;Skip if not Zero; this is a Databit
		goto	Stop_Bit		;Bit_Count=1, we have the Stopbit. Don't sample it into ShiftData
		
		rrf		ShiftData,1		;rotate right, schafft Platz fr ein neues bit. Result nach ShiftData		

		;re-start T0 for the next sampling time. don't activate IR
		movlw	Sample_Time	
		movwf	TMR0		

		;Datenbit schon mal setzen
		bsf		ShiftData,7		;set bit 7 to high
		;den Pegel des Eingangs-Signals testen		
		btfss	DataLine	 	;Pegel berprfen			
		;Das Bit wieder lschen, wenn der Pegel Low war
		bcf		ShiftData,7		;no, set Bit 7 low	
		movlw	0xA8			;prepare for Timer and Levelchange IR 
		goto	HouseKeeping_Exit
		
Stop_Bit
	;when we have read all bits, disable all IR and stop receiving
		movlw	0				;Disable Interrupts
		goto	HouseKeeping_Exit		

;------------Receiver mode: Send Binding Information--------------------------------------
Send_Bit
		btfsc	StartBit		;must we send the start bit?
		goto 	Switch_OFF		;yes, jump to Switch_OFF
		
Not_StartBit		
		
		bsf		STATUS,C		;fills up the Byte with 1's from top to bottom and produces the stop bit
		;jetzt ist das carry bit gesetzt und mit dem rrf wird es als high order bit reingeschoben
		rrf		INDF,F			;Das nchste Bit ins Carry shiften		
		;jetzt wurde Bit0 ins carry geschoben. Das wird jetzt berprft
		btfsc	STATUS,C		;Ist das Carry gesetzt?
		goto	Switch_ON		;Ist gesetzt, deshalb nach Switch_ON
Switch_OFF		
		bcf		DataLine		;Ist nicht gesetzt, deshalb OFF. Ausgang Low
		bcf		StartBit		;lsche Startbit Indikation
		goto	Cont
Switch_ON		
		bsf		DataLine		;Ausgang High
Cont				
		movlw	Full_Bit_Time 	
		movwf	TMR0			
		bcf		Send				;reset Indicator for outside routine
		movlw	0xA0	;GIE + T0IE			;Beim Senden nur Timer IR
;-------------------------------------------------------------------------------------
		

HouseKeeping_Exit
		;Wir setzen das Interrupt Control Register komplett neu auf
		movwf	INTCON

		movfw	GPIO
;		bcf		Channel1		;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!mein Synch Impuls frs Scope		

		swapf   status_temp,W	; retrieve copy of STATUS register
		movwf	STATUS			; restore pre-isr STATUS register contents Bank 0,1
		swapf   w_temp,F
		swapf   w_temp,W		; restore pre-isr W register contents
		retfie					; return from interrupt
;******************************************************************************
;********************* 19 Befehle lang ****************************************
;******************************************************************************

;						Ende der Interrupt Routine
;*******************************************************************************
;===============================================================================
;*******************************************************************************
;
;				  
;
;*******************************************************************************
; these first 4 instructions are not required if the internal oscillator is not used
;but they preserve the OscCal value
main	
		bsf     STATUS,RP0		; set file register bank to 1 
;		call    0x3FF			; retrieve factory calibration value
;		bsf     STATUS,RP0		; set file register bank to 1    
;		movwf   OSCCAL			; update register with factory cal value   Bank 1
		
		
;=====================Initialisierung===========================================

init
		bsf     STATUS,RP0		; set file register bank to 1 	
		clrf	PIE1			;   
		clrf	ANSEL			; I/O    			 	Bank 1		
		clrf	TRISIO			;   OUT  GP3					  Bank 1		
			
		movlw 	(0<<NOT_GPPU)|(0<<INTEDG)|(0<<T0CS)|(0<<T0SE)|(1<<PSA)|(0<<PS2)|(0<<PS1)|(0<<PS0)				
		movwf	OPTION_REG		;No Prescaler for Timer0, internal Clock, GPIO pullups ON			Bank 1	

		bcf		STATUS,RP0		;select bank 0	
										
		movlw	b'00000111'		;  		
		movwf	CMCON			;switch off comparator  					Bank 0		

		movlw	b'00000111'
		movwf	GPIO			;Dataline high, damit der receiver nicht von selbst in Bind mode geht, Alle  Ausgnge Low								Bank 0

		movlw	0x20
		movwf	FSR				;Bank 0,1
clr_loop
		clrf	INDF			;Bank 0,1
		incf	FSR,F			;Bank 0,1
		btfss	FSR,6			;Bank 0,1
		goto	clr_loop

		clrf	My_Temp
							
;  Bind 


		btfsc	BindLine		;
		goto	Normal_Op		;
;===============================================================
;          Bind Receiver with TX
;===============================================================
Request_Bind
		bcf		DataLine		;make dataline low to set receiver into bind mode
;wait 50-250 msec with Output GPIO2 low and then make GPIO2 high and an Input 
		call	Wait200m
		bsf		DataLine		;make it high, to terminate the "set to bindmode" 
		call	Setup_Registers_for_Receive		;setup of registers 
		call	Wait200m			;this delay is very important as obviously there is anyhow an edge that would trigger receive
		call	Wait200m
		call 	Receive_Bind_Info
		call	Schreiben			;Write to EEProm
;		bsf		LED					;switch on "Bind complete"	LED								Bank 0						
Term	
		nop							;Endless loop
		goto	Term		
;========================================================================
;========================================================================
		
;-------------------------------------------------------------------------------------------
;					Normal receive operation
;-------------------------------------------------------------------------------------------		

Normal_Op	
		call 	Lesen			;read the Binding Bytes from EEprom and put to Byte1 - Byte9   !ok geht!
								;wait 400 msec for the receiver to get powered on
		call	Wait200m	
		call	Wait200m	
		call	Setup_For_Send_BindInfo	;Timer0 free running with 24.4 usec(?) and data output via IR Handler !ok works!	
		bsf		Send_Bind				;Send Bind Information to receiver
		call	Send_Bind_Info		;wait and send binding information									! ok works !	
		bcf		Send_Bind
;******************************************************************************************
;*
;* 							   
;*
;******************************************************************************************
Cont_Receive	
		call	Setup_Registers_for_Receive		;setup registers and Timer0	
		call	Receive_Channel_Data		
		call	Output_Channel_Data				;Servo Kanal Impulse
		goto	Cont_Receive

;========================================================================
;========================================================================
;-----------------------------------------------------------------------
Wait500u
		clrf	Inner
I_Loop500
		nop
		nop
		nop					;385u
		nop					;450u
		nop					;510u
		decfsz	Inner,F	
		goto	I_Loop500
		return
;-----------------------------------------------------------------------
Wait30u	movlw	.10			
		movwf	Inner
I_Loop30
		decfsz	Inner,F	
		goto	I_Loop30
		return
;-----------------------------------------------------------------------
Wait60u	movlw	.77			;is setup for about 60usec delay
		movwf	Inner
I_Loop60
		decfsz	Inner,F	
		goto	I_Loop60
		return

;-----------------------------------------------------------------------		

Wait200m		
		movlw	.200	;setup for 200ms delay  
		movwf	Outer
O_Loop		
		movlw	.180
		movwf	Inner
I_Loop	
		movlw	.6
		movwf	IInner
II_Loop
		decfsz	IInner,F
		goto	II_Loop
		decfsz	Inner,F
		goto	I_Loop
		decfsz	Outer,F
		goto	O_Loop
		return		
;======================================================================			
		
Setup_For_Send_BindInfo

;don't allow Interrupts for Timer0 at this time
		clrf	PIR1			;   
		clrf	INTCON			;  						Bank 0,1
	
		bsf		STATUS,RP0		;select bank 1									
		bcf		TRISIO,2		;make GP2 an output								Bank 1	
		bsf		WPU,2			;activate weak pullup							Bank 1		
		bcf		STATUS,RP0		;select bank 1	

		movlw	Sample_Time		;Timer mit Wert 197 vorbesetzen. Damit luft er 24.4 usec bis zum Interrupt
		movwf	TMR0	
		movlw  (1<<GIE)|(0<<PEIE)|(1<<T0IE)|(0<<INTE)|(0<<GPIE)|(0<<T0IF)|(0<<INTF)|(0<<GPIF)
									;enable GIE + T0IE
		movwf	INTCON
		return

;======================================================================================================		
Setup_Registers_for_Receive		

;don't allow Interrupts for Timer0 at this time
		clrf	PIR1			;clear peripheral InterruptFlags Bank 0 ; nur sicherheitshalber
		
		bsf		STATUS,RP0		;select bank 1									
		bsf		TRISIO,2		;make GP2 an input	DataLine 					Bank 1
		bsf		IOC,IOC2		;enable GPIO2 for Interrupt on Change			Bank 1		
		bcf		STATUS,RP0		;select bank 0	
	
		return

;======================================================================================================		
;-------Read 10 (9) x 8Bit Binding Bytes--------------------------------------------------------------------
Receive_Bind_Info

		call	Wait200m			;allow Receiver to power on

;we receive one byte of FF that does belong to nothing, so put it into Temp and the next byte addressed is Byte1				
		movlw	Temp			;Adresse fr Schrottbyte vor Byte1
		movwf	FSR				;									Bank 0,1
;caused by the byte above, we have to set the bytecount one higher		
		movlw	.11				;wegen eines spikes auf der Dataline wird vor den Daten ein Pegel als Byte eingelesen
								;und nach temp gestellt. Deshalb mu hier der Bytecount eines hher sein
		movwf	Byte_Count
		
Cont_Bind_Info
		;clear possible IR flags and enable GIE and GPIE 
		;so it will wait for or continue with the next level change
		movlw	0x88			;jetzt nur Level Change Interrupt erlauben
		movwf	INTCON

		movlw	.10				;setup Bitcount. 9 passes, because the first pass is for the start bit
		movwf	Bit_Count

;---------------------------------		
;count the bits for one byte
Wait_For_Event1
		btfss	BitFound		;wait until the first level change IR or a timer0 Overrun IR occurres
		goto	Wait_For_Event1
		
		bcf		BitFound		;clear the found bit
		
		decfsz	Bit_Count,F		;count down, how many bits we have already
		goto	Wait_For_Event1			;not yet enough

;---------------------------------		
;now we have read in a full byte of 8 data and one stop bit

;clear all IF and disable IR
		clrf	INTCON
		
		;store the received byte
		movf	ShiftData,W		;ins W-Reg einlesen
		movwf	INDF			;nach 28 ff speichern					Bank 0,1
		incf	FSR,F			;										Bank 0,1
		;The delay between the bytes is 610 usec

		decfsz	Byte_Count,F		;all Bytes received?
		goto	Cont_Bind_Info	;no 
;alle 10 (9) Bytes have been read and are saved to Byte1 - Byte10	
		
		;all Interrupts disabled
		clrf	INTCON
		return		

;-----------------------------------------------------------------------------		
;==============================================================================
;store 10 (9) byte bind Daten to  EEProm 
Schreiben
		movlw	.10					;setup Bytecount
		movwf	Byte_Count			;nach Byte_Count speichern
		bcf		INTCON,7			;Interrupts ausschalten GIE				Bank 0,1 
		
		movlw	Byte1				;Data-adress for Byte1, where Data come from
		movwf	FSR					;									Bank 0,1
		movlw	.34					;
		movwf	To_Adrs				;Adresse of EEProm, to where data is going		

Again		
		call	EEProm_Write
		incf	FSR,F				;									Bank 0,1
		incf	To_Adrs,F

Wait								;Wait for EEProm Write to finish		
		btfss	PIR1,EEIF			;Weiter wenn InterruptFlag gesetzt, Schreiben ist beendet Bank 0
		goto	Wait				;Warten bis Schreiben beendet ist
		
		;Schreiben ist beendet
		bcf		PIR1,EEIF			;IR Flag lschen										Bank 0
		decfsz	Byte_Count,F		;all Bytes written?
		goto	Again				;No, next Byte
		return
				
;======================================================================

Send_Bind_Info		;send binding information 9 x 8bit bytes
;at least 9 Byte have tobe sent, 10 Bytes don't hurt
		bsf		Send_Bind				;Send Bind Information to receiver
		;copy first byte into Shift_Data
		bsf		Send			;set transmit mode for the interrupt routine
		bsf		StartBit		;to send the initial startbit
		movlw	.10
		movwf	Byte_Count
		movlw	Byte1
		movwf	FSR					;gets Adress of Byte1 			Bank 0,1

Next_Byte		

;Deshalb werden hier pro byte 13 bits gesendet. 1 Startbit, 8 Datenbits und 4 Stopbits
;we transmit 13 bits. 1 startbit, 8 databits, 4 stopbits

		movlw	.15					;Bitcount mit Start und mehreren Stop bits
		movwf	Bit_Count
		movf	INDF,W				;Daten von Byte1 in W						Bank 0,1
		
;------------------------------------
;darauf warten, da die Interrupt routine ein Bit auf die Leitung gestellt hat
Wait_Send							;Wait for Send to be switched off from IR Routine
		btfsc	Send
		goto	Wait_Send
;------------------------------------

		bsf		Send				;Indicator wieder ON setzen
		decfsz	Bit_Count,F			;alle bits durch?
		goto	Wait_Send					;Nein, nchstes Bit
		
		incf	FSR,F				;										Bank 0,1
		bsf		StartBit
		decfsz	Byte_Count,F			;Alle Bytes durch?
		
		goto	Next_Byte			;Nein,nchstes Byte
		bcf		Send_Bind		
		return
		
;============================================================================
;-------------------------------------------------------------------------		

Receive_Channel_Data		;receive 9 x 16bit bytes  1S, 16D, 1S
		;clear possible IR Flags
		movlw	0x88			;don't allow timer0 imterrupts at this time
		movwf	INTCON
		
		movlw	Byte1			; Byte1 
		movwf	FSR				;									Bank 0,1
		movlw	.18				;fill up 18 Bytes
		movwf	Byte_Count

Cont_Channel_Data
		movlw	.18				;setup Bitcount 1 Startbit, 16 Databits, 1 Stopbit
		movwf	Bit_Count

;---------------------------------		
;count the bits for one word
Wait_For_Event2
		btfss	BitFound		;wait until the first level change IR or a timer0 Overrun IR occurres
		goto	Wait_For_Event2
;---------------------
;when the bitcounter has been decremented from 18 to 10 we have received the first complete byte into shiftdata
		movlw	.10
		subwf	Bit_Count,W		;test bitcount = 10
		btfsc	STATUS,Z
		call	Save_Byte		;bitcount is 10, we have read the first byte
		
		bcf		BitFound		;das found bit lschen	
;when the bitcounter goes to 0, the second databyte is available in shiftdata
		decfsz	Bit_Count,F		;do we have one full word (1+16+1 Bit)?
		goto	Wait_For_Event2		;not yet
;--------------------		
		call	Save_Byte
		;after every second byte (one full word) we must take a short break
		call	Wait_Between
				
		movf	Byte_Count,F	;test bytecount for zero
		btfss	STATUS,Z		;Ist der Byte_Count=0 ?
		goto	Cont_Channel_Data	;no, go on read next word
		
		clrf	INTCON			;clear interrupts									Bank 0,1		
		return
		
		
;==========================================================================
Save_Byte	
;we have a byte, so let's save it		
		movf	ShiftData,W		;ins W-Reg einlesen
		movwf	INDF			;nach 28 ff speichern						Bank 0,1
		incf	FSR,F			;											Bank 0,1
		decf	Byte_Count,F
		return		
;=============================================================================
Wait_Between
;delay could probably be ignored, only setting the interrupt masks
		clrf	INTCON			;komplett lschen									Bank 0,1
		
;		call	Wait60u			;Wir haben das ganze Wort eingelesen und mssen warten bis das Stopbit vorbei ist
								;erst die nchste Flanke startet wieder das Einlesen
		
		movlw	0x88			;GIE + GPIE
		movwf	INTCON			;							Bank 0,1
		return
				
;===========================================================================================-		
		
Output_Channel_Data			;create PPM Sum

		clrf	INTCON			;Interrupts komplett

	;	call	CheckSum
	;	btfsc	Checksum_Ok			;if checksum is invalid, we skip copying of the channel data
	;	call	Copy_Ch_Data		;Data is correct, so use it
									;Data are now available in Cpy_Byte1  -  Cpy_Byte16

;	bsf		LED				; #Debag
		movlw	.8
		movwf	Byte_Count			;Number of channels to support (max 8)
		movlw	Byte1		 		;low byte channel1
;		movlw	Copy_Byte1			;low byte channel1
		movwf	FSR					;

		bcf		Channel1	
;						call	Wait30u
		nop
		nop
		nop
		nop
		nop
Next_Channel		

		clrf	Temp
		decf	Temp,F			
		movfw	INDF				
		subwf	Temp,W				
		movwf	TMR1L			
		incf	FSR,F				
		movfw	INDF				
		subwf	Temp,W				
		movwf	TMR1H			
		incf	FSR,F			
			
		movlw	0x21
		movwf 	T1CON				;enable Timer1 prescaler 4 : 1

		bsf		Channel1 	;on
Wait_Ch
		btfss	PIR1,0		;warten bis der Timer ausgetimed hat
		goto	Wait_Ch

		bcf		Channel1	;off
;						call	Wait30u

		clrf	T1CON
		bcf		PIR1,0		;reset IR Flag		
		
		
		decfsz	Byte_Count,F
		goto	Next_Channel

		bsf		Channel1
;	bcf		LED				; #Debag

		return				

;********************************************

Output_Channel_Data2		;   USART 115200

	decfsz  deb	;  
	return		;
	movlw	.50	;   50 
	movwf	deb	;

		movlw	.1
		movwf	Selected_Channel	;shifting bit that will select the physical output port
		movlw	.8
		movwf	Byte_Count			;Number of channels to support (max 8)
		movlw	Byte1
		movwf	FSR
Next_Channel2		
		movfw	INDF				;load low byte of a channel
		movwf	temp2	

		incf	FSR,F				;step to next byte adress
		movfw	INDF				;load high byte of a channel
		incf	FSR,F				;step to next byte adress
		movwf	Temp
	decf	Temp
	decf	Temp
	decf	Temp
		movlw	0x70				;ofset
		subwf	temp2
		btfss	STATUS,C		
		decf	Temp

		call Usart_OUT
		movfw	temp2
		movwf	Temp
		call Usart_OUT


		movlw	.60
		movwf	S_pause
S_p5	decfsz	S_pause
		goto	S_p5
		
		decfsz	Byte_Count,F
		goto	Next_Channel2

		return
;=============================================================================================
Usart_OUT
		bcf		LED				;   (Low)
		movlw	.8
		movwf	bits_usart

		movlw	.9
		movwf	S_pause
S_p1	decfsz	S_pause
		goto	S_p1
	;	nop
		movfw	Temp
		iorlw	b'00000010'		;    	Channel1
loop_US
		movwf	GPIO
		movlw	.9
		movwf	S_pause
S_p2	decfsz	S_pause
		goto	S_p2
	;	nop
		rrf		Temp
		movfw	Temp
		iorlw	b'00000010'		;    	Channel1
		decfsz	bits_usart		
		goto	loop_US

		bsf		LED				;   (High)
		movlw	.9
		movwf	S_pause
S_p3	decfsz	S_pause
		goto	S_p3
		return
;=============================================================================================
;================================================================================
;=============================================================================================
;===========================================================================================-		
;		 

Output_Channel_Data3			;create Channel Pulse for Servo	   4017 

		clrf	INTCON			;Interrupts komplett lschen
	
;		call	CheckSum
;		btfsc	Checksum_Ok			;if checksum is invalid, we skip copying of the channel data
;		call	Copy_Ch_Data		;Data is correct, so use it

		movlw	.8
		movwf	Byte_Count			;Number of channels to support (max 8)
		movlw	Byte1				;low byte channel1
		movwf	FSR

		bcf		Channel1		
Next_Channel3		
		clrf	Temp
		decf	Temp,F
		movfw	INDF				;load low byte of a channel
		subwf	Temp,W				;subtract from 0
		movwf	TMR1L				;set timer low byte
		incf	FSR,F				;step to next byte adress
		movfw	INDF				;load high byte of a channel
		subwf	Temp,W				;subtract from 0
		movwf	TMR1H				;set timer high byte
		incf	FSR,F				;step to next byte adress
		
		movlw	0x21
		movwf 	T1CON				;enable Timer1 prescaler 4 : 1
		
		bsf		LED

Wait_Ch3
		btfss	PIR1,0				;warten bis der Timer ausgetimed hat
		goto	Wait_Ch3
		bcf		LED		

;		bcf		Channel1		

		clrf	T1CON
		bcf		PIR1,0		;reset IR Flag		

		decfsz	Byte_Count,F
		goto	Next_Channel3

		bsf		Channel1

		return

;========================================================================================
;========================================================================================
;
CheckSum

;Die ersten 8 Worte sind als 16bit Worte zu addieren. Das nchste 16Bit Wort ist die Prfumme
;Die Daten kommen mit 8 Bit low und dann 8 Bit high
;Datenstream: EE05 EA05 3D06 0306 D205 FB03 DC05 DC05 9D2D
;Zahlenwerte: 05EE 05EA 063D 0603 05D2 03FB 05DC 05DC 2D9D
;Prfsummenrechnung:  05EE + 05EA+ + 063D + 0603 + 05D2 + 03FB + 05DC + 05DC = 2D9D
;Kanalimpulslngen:   1518   1514    1597   1539   1490   1019   1500   1500  [usec]

;for the checksum, we must add 16bit words. We add up the low order bytes in W and collect the carry bits in Cksum_High
;we exit, if checksum low miscompares.
;we take Cksum_High and add up the high order bytes and compare with checksum high

		bcf		Checksum_Ok			;ersteinmal ausmachen, wenn Prfsumme ok war, wird es wieder angemacht
		clrf	Cksum_High
		movlw	Byte3				;Adresse fr 2. lo Byte
		movwf	FSR				;									Bank 0,1
		movlw	.7					;16 Byte Channel Daten vergleichen mit high,low Prfsumme, 1 Byte direkt nach W geladen
		movwf	Byte_Count
		movf	Byte1,W				;Daten aus Byte1 in W stellen
Sum_Up_Lo		
		addwf	INDF,W				;addiere Daten von Byte1 - 15 in W						Bank 0,1
		btfsc	STATUS,C			;Wenn es beim Addieren einen bertrag gegeben hat, dann mu Checksum_High incremented werden
		incf	Cksum_High,F		
		incf	FSR,F
		incf	FSR,F
		decfsz	Byte_Count,F		;schon alle Bytes empfangen?		
		goto	Sum_Up_Lo
;Checksum Low vergleichen		
;In Byte 
		;in W steht nun Checksum low, FSR steht schon auf Checksum-low
		subwf	INDF,W				;Registerwert von W abziehen, Ergebnis in W.
									;Wenn das Ergebnis Null ist, dann war Checksum_Low ok
		btfss	STATUS,Z			;Zero gesetzt wenn ok									
		goto	Fail				;Ergebnis ist nicht Null, also war Prfsumme_low unterschiedlich
;----------------------		
		movlw	.8					;16 Byte Channel Daten vergleichen mit high,low Prfsumme, 1 Byte direkt nach W geladen
		movwf	Byte_Count
		movlw	Byte2				;Adresse fr 1. hi Byte
		movwf	FSR				;									Bank 0,1
		movf	Cksum_High,W		;aus der Addition der lo bytes aufgelaufener Hi-byte Wert in W stellen
Sum_Up_Hi		
		addwf	INDF,W				;addiere Daten von Byte2 - 16 in W						Bank 0,1
		incf	FSR,F
		incf	FSR,F
		decfsz	Byte_Count,F		;schon alle Bytes empfangen?		
		goto	Sum_Up_Hi
;Checksum Low vergleichen		
;In Byte 
		;in W steht nun Checksum low, FSR steht schon auf Checksum-low
		subwf	INDF,W				;Registerwert von W abziehen, Ergebnis in W.
									;Wenn das Ergebnis Null ist, dann war Checksum_Low ok
		btfss	STATUS,Z			;Zero gesetzt wenn ok									
		goto	Fail				;Ergebnis ist nicht Null, also war Prfsumme_low unterschiedlich


;Prfsumme ok, wir knnen die Daten kopieren damit wir sie auch bei einer Strung weiter zur Verfgung haben		
;wobei ja der Empfnger ohnehin bei Signalausfall etwa 2 sekunden lang Daten sendet.
		bsf		Checksum_Ok	
			
;		bcf		LED
		return
	
Fail	
;		bsf		LED
		return		
		
;=============================================================================		
;Die Daten von Byte1 - Byte16 werden nach Copy_Byte1 ff kopiert
Copy_Ch_Data
		movlw	.16
		movwf	Byte_Count
		
		movlw	Copy_Byte1
		movwf	To_Adrs
		
		movlw	Byte1
		movwf	From_Adrs		
Copy_Cont		
		movf	From_Adrs,W		;From adresse nach W
		movwf	FSR				;und von da nach FSR
		movf 	INDF,W			;Daten von from adresse lesen
		movwf	Temp			;und nach Temp stellen
		
		movf	To_Adrs,W			;die to adresse holen
		movwf	FSR				;nach FSR stellen
		movf 	Temp,W			;die zu movenden daten von Temp holen 
		movwf	INDF			;und nach To stellen
		incf	From_Adrs,F		;from adresse hochsteppen
		incf	To_Adrs,F		;to adresse hochsteppen
		decfsz	Byte_Count,F	;den Bytecounter runterzhlen
		goto	Copy_Cont
		return
		
;=============================================================================			
;=============================================================================				
EEProm_Write
		bsf		STATUS,RP0			;BAnk 1 selektieren					Bank 0,1
		movf	To_Adrs,W			;die To-Adresse holen
		movwf	EEADR				;Adresse laden			Bank 1
		
		movf	INDF,W				;von 0x28 ff nach W holen			Bank 0,1		
		movwf	EEDATA				;Daten laden			Bank 1
		bsf		EECON1,WREN			;Schreiben erlauben		Bank 1
		movlw	55h
		movwf	EECON2				;						Bank 1
		movlw	0xAA				
		movwf	EECON2				;						Bank 1
		bsf		EECON1,WR			;schreiben starten		Bank 1
		bcf		STATUS,RP0			;select bank 0					Bank 0,1
		return				
;===============================================================================

;-----------------------------------------------------------------------------
;Bind Daten aus EEProm lesen und nach Byte1 - Byte10 schreiben
Lesen
		movlw	.10					;setup Bytecount
		movwf	Byte_Count			;nach Byte_Count speichern
		movlw	.34 				;Startadresse im EEprom wo Daten  herkommen
		movwf	From_Adrs				;									Bank 0,1
		movlw	Byte1				;Daten-Adresse fr Byte1, wo Daten hinsollen
		movwf	FSR					;Adresse wo Daten hinsollen				Bank 0,1

L_Again		
		call	EEProm_Read
		incf	From_Adrs,F
		incf	FSR,F				;									Bank 0,1
		decfsz	Byte_Count,F		;schon alle Bytes gelesen?
		goto	L_Again				;Nein, nchstes Byte lesen
		return

;=================================================================================
EEProm_Read
		bsf		STATUS,RP0			;select Bank 1						Bank 0,1
		movf	From_Adrs,W				;hole die source adresse
		movwf	EEADR				;holds Adress to be read			Bank 1
		bsf		EECON1,RD			;Start EE Read						Bank 1
		movf	EEDATA,W			;									Bank 1
		movwf	INDF				;speichere nach target adresse (Byte1 - Byte9)	Bank 0,1
		bcf		STATUS,RP0			;select Bank 0						Bank 0,1
		return



;CODE	0x03FF
;		retlw  35 ;     RC 
;=================================================================================   
; initialize eeprom locations

EE		CODE	0x2100
		DE	"Reinhard Strauch 2009; Decoder "
		DE	Prog_Level				;Programmversion
		DE " "
		DE 0x77,0x62,0x45,0x87,0x54,0x35,0x53,0x55,0x85,0x18	;mi bind cod
		DE "              "				;only for Test
		;10 Bytes aus Bind procedure vom Sender
		END                       ; directive 'end of program'
